# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import six
from webob import exc

from oslo_serialization import jsonutils

from senlin.api.common import util
from senlin.api.middleware import fault
from senlin.api.openstack.v1 import receivers
from senlin.common import exception as senlin_exc
from senlin.common import policy
from senlin.rpc import client as rpc_client
from senlin.tests.unit.api import shared
from senlin.tests.unit.common import base


@mock.patch.object(policy, 'enforce')
class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
    def setUp(self):
        super(ReceiverControllerTest, self).setUp()

        class DummyConfig(object):
            bind_port = 8778

        cfgopts = DummyConfig()
        self.controller = receivers.ReceiverController(options=cfgopts)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_index_normal(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)
        req = self._get('/receivers')

        engine_resp = [
            {
                u'id': u'aaaa-bbbb-cccc',
                u'name': u'test-receiver',
                u'type': u'webhook',
                u'user': u'admin',
                u'project': u'123456abcd3555',
                u'domain': u'default',
                u'cluster_id': u'FAKE_CLUSTER',
                u'action': u'test-action',
                u'actor': {
                    u'user_id': u'test-user-id',
                    u'password': u'test-pass',
                },
                u'created_time': u'2015-02-24T19:17:22Z',
                u'params': {},
                'channel': {
                    'alarm_url': 'http://somewhere/on/earth',
                },
            }
        ]

        mock_call.return_value = engine_resp
        obj = mock.Mock()
        mock_parse.return_value = obj

        result = self.controller.index(req)

        self.assertEqual(engine_resp, result['receivers'])
        mock_parse.assert_called_once_with(
            'ReceiverListRequest', req, mock.ANY)
        mock_call.assert_called_with(req.context, 'receiver_list', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_index_whitelists_params(self, mock_call,
                                              mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)
        marker = 'cac6d9c1-cb4e-4884-ba2a-3cbc72d84aaf'
        params = {
            'limit': 20,
            'marker': marker,
            'sort': 'name:desc',
            'name': 'receiver01',
            'type': 'webhook',
            'cluster_id': '123abc',
            'action': 'CLUSTER_RESIZE',
            'user': 'user123'
        }
        req = self._get('/receivers', params=params)

        mock_call.return_value = []
        obj = mock.Mock()
        mock_parse.return_value = obj

        result = self.controller.index(req)

        self.assertEqual([], result['receivers'])
        mock_parse.assert_called_once_with(
            'ReceiverListRequest', req,
            {
                'sort': 'name:desc',
                'name': ['receiver01'],
                'action': ['CLUSTER_RESIZE'],
                'limit': '20',
                'marker': marker,
                'cluster_id': ['123abc'],
                'type': ['webhook'],
                'project_safe': True,
                'user': ['user123']
            })
        mock_call.assert_called_with(req.context, 'receiver_list', mock.ANY)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_index_whitelists_invalid_params(self, mock_call,
                                                      mock_parse,
                                                      mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)
        params = {
            'balrog': 'you shall not pass!'
        }
        req = self._get('/receivers', params=params)

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.index, req)

        self.assertEqual("Invalid parameter balrog", six.text_type(ex))
        self.assertFalse(mock_parse.called)
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_index_invalid_type(self, mock_call,
                                         mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)
        params = {'type': 'bogus'}
        req = self._get('/receivers', params=params)

        mock_parse.side_effect = exc.HTTPBadRequest("bad param")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.index, req)

        self.assertEqual("bad param", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverListRequest', req,
            {
                'type': ['bogus'],
                'project_safe': True
            })
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_index_invalid_action(self, mock_call,
                                           mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)

        params = {'action': 'bogus'}
        req = self._get('/receivers', params=params)

        mock_parse.side_effect = exc.HTTPBadRequest("bad param")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.index, req)

        self.assertEqual("bad param", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverListRequest', req,
            {
                'action': ['bogus'],
                'project_safe': True
            })
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_index_limit_non_int(self, mock_call,
                                          mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)

        params = {'limit': 'abc'}
        req = self._get('/receivers', params=params)

        mock_parse.side_effect = exc.HTTPBadRequest("bad param")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.index, req)

        self.assertEqual("bad param", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverListRequest', req,
            {
                'limit': 'abc',
                'project_safe': True
            })
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_index_invalid_sort(self, mock_call,
                                         mock_parse, mock_enforce):

        self._mock_enforce_setup(mock_enforce, 'index', True)

        params = {'sort': 'bogus:foo'}
        req = self._get('/receivers', params=params)

        mock_parse.side_effect = exc.HTTPBadRequest("bad param")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.index, req)

        self.assertEqual("bad param", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverListRequest', req,
            {
                'sort': 'bogus:foo',
                'project_safe': True
            })
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_index_global_project(self, mock_call,
                                           mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)

        params = {'global_project': True}
        req = self._get('/receivers', params=params)

        mock_call.return_value = []
        obj = mock.Mock()
        mock_parse.return_value = obj

        result = self.controller.index(req)

        self.assertEqual([], result['receivers'])
        mock_parse.assert_called_once_with(
            'ReceiverListRequest', req, {'project_safe': False})
        mock_call.assert_called_once_with(
            req.context, 'receiver_list', obj)

    def test_receiver_index_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', False)
        req = self._get('/receivers')

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.index, req)
        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_create_success(self, mock_call,
                                     mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'create', True)
        body = {
            'receiver': {
                'name': 'test_receiver',
                'type': 'webhook',
                'cluster_id': 'FAKE_ID',
                'action': 'CLUSTER_RESIZE',
                'actor': {
                    'user_id': 'test_user_id',
                    'password': 'test_pass',
                },
                'params': {
                    'test_param': 'test_value'
                },
            }
        }

        engine_response = {
            'id': 'xxxx-yyyy-zzzz',
            'name': 'test_receiver',
            'type': 'webhook',
            'cluster_id': 'FAKE_ID',
            'action': 'CLUSTER_RESIZE',
            'actor': {
                'user_id': 'test_user_id',
                'password': 'test_pass',
            },
            'params': {
                'test_param': 'test_value'
            },
            'channel': {
                'alarm_url': 'http://somewhere/on/earth',
            },
        }

        req = self._post('/receivers', jsonutils.dumps(body))
        mock_call.return_value = engine_response
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller.create(req, body=body)

        self.assertEqual(engine_response, resp['receiver'])
        mock_parse.assert_called_once_with(
            'ReceiverCreateRequest', req, body, 'receiver')
        mock_call.assert_called_with(
            req.context, 'receiver_create', obj.receiver)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_create_with_bad_body(self, mock_call, mock_parse,
                                           mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'create', True)
        body = {'name': 'test_receiver'}

        req = self._post('/receivers', jsonutils.dumps(body))

        mock_parse.side_effect = exc.HTTPBadRequest("bad param")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.create,
                               req, body=body)

        self.assertEqual("bad param", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverCreateRequest', req, body, 'receiver')
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_create_missing_required_field(self, mock_call,
                                                    mock_parse,
                                                    mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'create', True)
        body = {
            'receiver': {
                'name': 'test_receiver',
                'cluster_id': 'FAKE_CLUSTER',
                'action': 'CLUSTER_RESIZE',
            }
        }

        req = self._post('/receivers', jsonutils.dumps(body))

        mock_parse.side_effect = exc.HTTPBadRequest("miss type")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.create,
                               req, body=body)

        self.assertEqual("miss type", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverCreateRequest', req, body, 'receiver')
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_create_with_bad_type(self, mock_call,
                                           mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'create', True)
        r_type = 'unsupported'
        body = {
            'receiver': {
                'name': 'test_receiver',
                'type': r_type,
                'cluster_id': 'FAKE_CLUSTER',
                'action': 'CLUSTER_RESIZE',
            }
        }

        req = self._post('/receivers', jsonutils.dumps(body))

        mock_parse.side_effect = exc.HTTPBadRequest("bad type")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.create,
                               req, body=body)

        self.assertEqual("bad type", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverCreateRequest', req, body, 'receiver')
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_create_illegal_action(self, mock_call,
                                            mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'create', True)
        action = 'illegal_action'
        body = {
            'receiver': {
                'name': 'test_receiver',
                'type': 'webhook',
                'cluster_id': 'FAKE_CLUSTER',
                'action': action,
            }
        }
        req = self._post('/receivers', jsonutils.dumps(body))

        mock_parse.side_effect = exc.HTTPBadRequest("bad action")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.create,
                               req, body=body)

        self.assertEqual("bad action", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverCreateRequest', req, body, 'receiver')
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_get_normal(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'get', True)
        wid = 'aaaa-bbbb-cccc'
        req = self._get('/receivers/%(receiver_id)s' % {'receiver_id': wid})

        engine_resp = {
            u'id': u'aaaa-bbbb-cccc',
            u'name': u'test-receiver',
            u'type': u'webhook',
            u'user': u'admin',
            u'project': u'123456abcd3555',
            u'domain': u'default',
            u'cluster_id': u'FAKE_CLUSTER',
            u'action': u'test-action',
            u'actor': {
                u'user_id': u'test-user-id',
                u'password': u'test-pass',
            },
            u'created_time': u'2015-02-24T19:17:22Z',
            u'params': {},
            u'channel': {
                u'alarm_url': u'http://somewhere/on/earth',
            }
        }

        mock_call.return_value = engine_resp
        obj = mock.Mock()
        mock_parse.return_value = obj

        result = self.controller.get(req, receiver_id=wid)

        self.assertEqual(engine_resp, result['receiver'])
        mock_parse.assert_called_once_with(
            'ReceiverGetRequest', req, {'identity': wid})
        mock_call.assert_called_with(req.context, 'receiver_get', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_get_not_found(self, mock_call, mock_parse,
                                    mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'get', True)
        wid = 'non-existent-receiver'
        req = self._get('/receivers/%(receiver_id)s' % {'receiver_id': wid})

        error = senlin_exc.ResourceNotFound(type='receiver', id=wid)
        mock_call.side_effect = shared.to_remote_error(error)

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.get,
                                              req, receiver_id=wid)

        self.assertEqual(404, resp.json['code'])
        self.assertEqual('ResourceNotFound', resp.json['error']['type'])

    def test_receiver_get_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'get', False)
        wid = 'non-existent-receiver'
        req = self._get('/receivers/%(receiver_id)s' % {'receiver_id': wid})

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.get,
                                              req, receiver_id=wid)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_update_normal(self, mock_call, mock_parse,
                                    mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', True)
        wid = 'aaaa-bbbb-cccc'
        body = {
            'receiver': {
                'name': 'receiver-2',
                'params': {
                    'count': 10,
                }
            }
        }

        req = self._put('/receivers/%(receiver_id)s' % {'receiver_id': wid},
                        jsonutils.dumps(body))

        engine_response = {
            u'id': wid,
            u'name': u'receiver-2',
            u'created_time': u'2015-02-25T16:20:13Z',
            u'updated_time': None,
            u'params': {u'count': 10},
        }

        mock_call.return_value = engine_response
        obj = mock.Mock()
        mock_parse.return_value = obj

        result = self.controller.update(req, receiver_id=wid, body=body)

        self.assertEqual(engine_response, result['receiver'])
        mock_parse.assert_called_once_with(
            'ReceiverUpdateRequest', req, mock.ANY)
        mock_call.assert_called_once_with(
            req.context, 'receiver_update', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_update_no_body(self, mock_call, mock_parse,
                                     mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', True)
        wid = 'aaaa-bbbb-cccc'
        body = {'foo': 'bar'}
        req = self._put('/receivers/%(receiver_id)s' % {'receiver_id': wid},
                        jsonutils.dumps(body))

        mock_parse.side_effect = exc.HTTPBadRequest("bad body")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.update,
                               req, receiver_id=wid, body=body)

        self.assertEqual("Malformed request data, missing 'receiver' key "
                         "in request body.", six.text_type(ex))
        self.assertFalse(mock_parse.called)
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_update_no_name(self, mock_call, mock_parse,
                                     mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', True)
        wid = 'aaaa-bbbb-cccc'
        body = {
            'receiver': {'params': {'count': 10}}
        }

        req = self._put('/receivers/%(receiver_id)s' % {'receiver_id': wid},
                        jsonutils.dumps(body))
        mock_call.return_value = {}
        obj = mock.Mock()
        mock_parse.return_value = obj

        result = self.controller.update(req, receiver_id=wid, body=body)

        self.assertEqual({}, result['receiver'])
        mock_parse.assert_called_once_with(
            'ReceiverUpdateRequest', req, mock.ANY)
        mock_call.assert_called_once_with(
            req.context, 'receiver_update', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_update_with_unexpected_field(self, mock_call,
                                                   mock_parse,
                                                   mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', True)
        wid = 'aaaa-bbbb-cccc'
        body = {
            'receiver': {
                'name': 'receiver-2',
                'params': {'count': 10},
            }
        }
        req = self._put('/receivers/%(receiver_id)s' % {'receiver_id': wid},
                        jsonutils.dumps(body))

        mock_parse.side_effect = exc.HTTPBadRequest("bad param")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.update,
                               req, receiver_id=wid, body=body)

        self.assertEqual("bad param", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverUpdateRequest', req, mock.ANY)
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_update_not_found(self, mock_call, mock_parse,
                                       mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', True)
        wid = 'non-existent-receiver'
        body = {
            'receiver': {
                'name': 'receiver-2',
                'params': {'count': 10},
            }
        }
        req = self._put('/receivers/%(receiver_id)s' % {'receiver_id': wid},
                        jsonutils.dumps(body))

        error = senlin_exc.ResourceNotFound(type='webhook', id=wid)
        mock_call.side_effect = shared.to_remote_error(error)

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.update,
                                              req, receiver_id=wid,
                                              body=body)

        self.assertEqual(404, resp.json['code'])
        self.assertEqual('ResourceNotFound', resp.json['error']['type'])

    def test_receiver_update_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', False)
        wid = 'aaaa-bbbb-cccc'
        body = {
            'receiver': {'name': 'receiver-2', 'spec': {'param5': 'value5'}},
        }
        req = self._put('/receivers/%(receiver_id)s' % {'receiver_id': wid},
                        jsonutils.dumps(body))

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.update,
                                              req, profile_id=wid,
                                              body=body)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_delete_success(self, mock_call, mock_parse,
                                     mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'delete', True)
        wid = 'aaaa-bbbb-cccc'
        req = self._delete('/receivers/%(receiver_id)s' % {'receiver_id': wid})

        obj = mock.Mock()
        mock_parse.return_value = obj

        self.assertRaises(exc.HTTPNoContent,
                          self.controller.delete, req, receiver_id=wid)
        mock_parse.assert_called_once_with(
            'ReceiverDeleteRequest', req, {'identity': wid})
        mock_call.assert_called_once_with(
            req.context, 'receiver_delete', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_delete_err_malformed_receiver_id(self, mock_call,
                                                       mock_parse,
                                                       mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'delete', True)
        wid = {'k1': 'v1'}
        req = self._delete('/receivers/%(receiver_id)s' % {'receiver_id': wid})

        mock_parse.side_effect = exc.HTTPBadRequest("bad identity")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.delete, req,
                               receiver_id=wid)
        self.assertEqual("bad identity", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverDeleteRequest', req, {'identity': wid})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_delete_not_found(self, mock_call, mock_parse,
                                       mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'delete', True)
        wid = 'aaaa-bbbb-cccc'
        req = self._delete('/receivers/%(receiver_id)s' % {'receiver_id': wid})

        error = senlin_exc.ResourceNotFound(type='receiver', id=wid)
        mock_call.side_effect = shared.to_remote_error(error)

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.delete,
                                              req, receiver_id=wid)

        self.assertEqual(404, resp.json['code'])
        self.assertEqual('ResourceNotFound', resp.json['error']['type'])

    def test_receiver_delete_err_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'delete', False)
        wid = 'aaaa-bbbb-cccc'
        req = self._delete('/receivers/%(receiver_id)s' % {'receiver_id': wid})

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.delete,
                                              req, receiver_id=wid)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_notify_success(self, mock_call, mock_parse,
                                     mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'notify')
        wid = 'aaaa-bbbb-cccc'
        req = self._post('/receivers/%(receiver_id)s/notify' % {
            'receiver_id': wid}, None)

        obj = mock.Mock()
        mock_parse.return_value = obj

        self.assertRaises(exc.HTTPNoContent,
                          self.controller.notify, req, receiver_id=wid)
        mock_parse.assert_called_once_with(
            'ReceiverNotifyRequest', req, {'identity': wid})
        mock_call.assert_called_with(req.context, 'receiver_notify', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_notify_err_malformed_receiver_id(self, mock_call,
                                                       mock_parse,
                                                       mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'notify', True)
        wid = {'k1': 'v1'}
        req = self._post('/receivers/%(receiver_id)s' % {'receiver_id': wid},
                         None)

        mock_parse.side_effect = exc.HTTPBadRequest("bad identity")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.notify, req,
                               receiver_id=wid)
        self.assertEqual("bad identity", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ReceiverNotifyRequest', req, {'identity': wid})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_receiver_notify_not_found(self, mock_call, mock_parse,
                                       mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'notify', True)
        wid = 'aaaa-bbbb-cccc'
        req = self._post('/receivers/%(receiver_id)s/notify' % {
            'receiver_id': wid}, None)

        error = senlin_exc.ResourceNotFound(type='receiver', id=wid)
        mock_call.side_effect = shared.to_remote_error(error)

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.notify,
                                              req, receiver_id=wid)

        self.assertEqual(404, resp.json['code'])
        self.assertEqual('ResourceNotFound', resp.json['error']['type'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Policy Engine For Senlin
"""

# from oslo_concurrency import lockutils
from oslo_config import cfg
from oslo_policy import policy

from senlin.common import exception
from senlin.common import policies

POLICY_ENFORCER = None
CONF = cfg.CONF


# @lockutils.synchronized('policy_enforcer', 'senlin-')
def _get_enforcer(policy_file=None, rules=None, default_rule=None):

    global POLICY_ENFORCER

    if POLICY_ENFORCER is None:
        POLICY_ENFORCER = policy.Enforcer(CONF,
                                          policy_file=policy_file,
                                          rules=rules,
                                          default_rule=default_rule)
        POLICY_ENFORCER.register_defaults(policies.list_rules())
    return POLICY_ENFORCER


def enforce(context, rule, target, do_raise=True, *args, **kwargs):

    enforcer = _get_enforcer()
    credentials = context.to_dict()
    target = target or {}
    if do_raise:
        kwargs.update(exc=exception.Forbidden)

    return enforcer.enforce(rule, target, credentials, do_raise,
                            *args, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from senlin.common import consts
from senlin.engine.actions import base as ab
from senlin.engine.actions import cluster_action as ca
from senlin.engine import cluster as cm
from senlin.engine import dispatcher
from senlin.objects import action as ao
from senlin.objects import dependency as dobj
from senlin.objects import node as no
from senlin.tests.unit.common import base
from senlin.tests.unit.common import utils


@mock.patch.object(cm.Cluster, 'load')
class ClusterReplaceNodesTest(base.SenlinTestCase):

    def setUp(self):
        super(ClusterReplaceNodesTest, self).setUp()
        self.ctx = utils.dummy_context()

    @mock.patch.object(ao.Action, 'update')
    @mock.patch.object(ab.Action, 'create')
    @mock.patch.object(no.Node, 'get')
    @mock.patch.object(dobj.Dependency, 'create')
    @mock.patch.object(dispatcher, 'start_action')
    @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
    def test_do_replace_nodes(self, mock_wait, mock_start, mock_dep,
                              mock_get_node, mock_action, mock_update,
                              mock_load):
        cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=10)
        mock_load.return_value = cluster

        action = ca.ClusterAction(cluster.id, 'CLUSTER_ACTION', self.ctx)
        action.id = 'CLUSTER_ACTION_ID'
        action.inputs = {'O_NODE_1': 'R_NODE_1'}
        action.outputs = {}

        origin_node = mock.Mock(id='O_NODE_1', cluster_id='CLUSTER_ID',
                                ACTIVE='ACTIVE', status='ACTIVE')
        replace_node = mock.Mock(id='R_NODE_1', cluster_id='',
                                 ACTIVE='ACTIVE', status='ACTIVE')
        mock_get_node.side_effect = [origin_node, replace_node]
        mock_action.side_effect = ['NODE_LEAVE_1', 'NODE_JOIN_1']
        mock_wait.return_value = (action.RES_OK, 'Free to fly!')

        # do the action
        res_code, res_msg = action.do_replace_nodes()

        # assertions
        self.assertEqual(action.RES_OK, res_code)
        self.assertEqual('Completed replacing nodes.', res_msg)

        mock_get_node.assert_has_calls([
            mock.call(action.context, 'O_NODE_1'),
            mock.call(action.context, 'R_NODE_1')])
        mock_load.assert_called_once_with(
            action.context,
            'CLUSTER_ID')
        mock_action.assert_has_calls([
            mock.call(action.context, 'O_NODE_1', 'NODE_LEAVE',
                      name='node_leave_O_NODE_1',
                      cause='Derived Action'),
            mock.call(action.context, 'R_NODE_1', 'NODE_JOIN',
                      name='node_join_R_NODE_1',
                      cause='Derived Action',
                      inputs={'cluster_id': 'CLUSTER_ID'})])

        mock_dep.assert_has_calls([
            mock.call(action.context,
                      ['NODE_JOIN_1'],
                      'CLUSTER_ACTION_ID'),
            mock.call(action.context,
                      ['NODE_JOIN_1'],
                      'NODE_LEAVE_1')])

        mock_update.assert_has_calls([
            mock.call(action.context,
                      'NODE_JOIN_1',
                      {'status': 'READY'}),
            mock.call(action.context,
                      'NODE_LEAVE_1',
                      {'status': 'READY'})])
        mock_start.assert_called_once_with()

        mock_wait.assert_called_once_with()

        cluster.remove_node.assert_called_once_with(origin_node)
        cluster.add_node.assert_called_once_with(replace_node)
        cluster.eval_status.assert_called_once_with(
            action.context, consts.CLUSTER_REPLACE_NODES)

    @mock.patch.object(no.Node, 'get')
    def test_do_replace_nodes_original_not_found(self, mock_get_node,
                                                 mock_load):
        action = ca.ClusterAction('ID', 'CLUSTER_ACTION', self.ctx)
        action.inputs = {'ORIGIN_NODE': 'REPLACE_NODE'}
        origin_node = None
        replace_node = mock.Mock(id='REPLACE_NODE', cluster_id='',
                                 ACTIVE='ACTIVE', status='ACTIVE')
        mock_get_node.side_effect = [origin_node, replace_node]
        # do the action
        res_code, res_msg = action.do_replace_nodes()

        # assertions
        self.assertEqual(action.RES_ERROR, res_code)
        self.assertEqual('Original node ORIGIN_NODE not found.',
                         res_msg)

    @mock.patch.object(no.Node, 'get')
    def test_do_replace_nodes_replacement_not_found(self, mock_get_node,
                                                    mock_load):
        action = ca.ClusterAction('ID', 'CLUSTER_ACTION', self.ctx)
        action.inputs = {'ORIGIN_NODE': 'REPLACE_NODE'}
        origin_node = mock.Mock(id='ORIGIN_NODE', cluster_id='CLUSTER_ID',
                                ACTIVE='ACTIVE', status='ACTIVE')
        replace_node = None
        mock_get_node.side_effect = [origin_node, replace_node]
        # do the action
        res_code, res_msg = action.do_replace_nodes()

        # assertions
        self.assertEqual(action.RES_ERROR, res_code)
        self.assertEqual('Replacement node REPLACE_NODE not found.',
                         res_msg)

    @mock.patch.object(no.Node, 'get')
    def test_do_replace_nodes_not_a_member(self, mock_get_node,
                                           mock_load):
        cluster = mock.Mock(id='FAKE_CLUSTER')
        mock_load.return_value = cluster

        action = ca.ClusterAction(cluster.id, 'CLUSTER_ACTION', self.ctx)
        action.inputs = {'ORIGIN_NODE': 'REPLACE_NODE'}

        origin_node = mock.Mock(id='ORIGIN_NODE', cluster_id='')
        mock_get_node.return_value = origin_node
        # do action
        res_code, res_msg = action.do_replace_nodes()

        # assertions
        self.assertEqual(action.RES_ERROR, res_code)
        self.assertEqual('Node ORIGIN_NODE is not a member of the '
                         'cluster FAKE_CLUSTER.', res_msg)

    @mock.patch.object(no.Node, 'get')
    def test_do_replace_nodes_node_already_member(self, mock_get_node,
                                                  mock_load):
        cluster = mock.Mock(id='FAKE_CLUSTER')
        mock_load.return_value = cluster

        action = ca.ClusterAction(cluster.id, 'CLUSTER_ACTION', self.ctx)
        action.inputs = {'ORIGIN_NODE': 'REPLACE_NODE'}

        replace_node = mock.Mock(id='REPLACE_NODE',
                                 cluster_id='FAKE_CLUSTER')
        mock_get_node.return_value = replace_node

        # do it
        res_code, res_msg = action.do_replace_nodes()

        # assertions
        self.assertEqual(action.RES_ERROR, res_code)
        self.assertEqual('Node REPLACE_NODE is already owned by cluster '
                         'FAKE_CLUSTER.', res_msg)

    @mock.patch.object(no.Node, 'get')
    def test_do_replace_nodes_in_other_cluster(self, mock_get_node,
                                               mock_load):
        cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=10)
        mock_load.return_value = cluster

        action = ca.ClusterAction(cluster.id, 'CLUSTER_ACTION', self.ctx)
        action.id = 'CLUSTER_ACTION_ID'
        action.inputs = {'ORIGIN_NODE': 'REPLACE_NODE'}
        action.outputs = {}

        origin_node = mock.Mock(id='ORIGIN_NODE', cluster_id='CLUSTER_ID',
                                ACTIVE='ACTIVE', status='ACTIVE')
        replace_node = mock.Mock(id='REPLACE_NODE', cluster_id='FAKE_CLUSTER',
                                 ACTIVE='ACTIVE', status='ACTIVE')
        mock_get_node.side_effect = [origin_node, replace_node]

        # do it
        res_code, res_msg = action.do_replace_nodes()

        # assertions
        self.assertEqual(action.RES_ERROR, res_code)
        self.assertEqual('Node REPLACE_NODE is already owned by cluster '
                         'FAKE_CLUSTER.', res_msg)

    @mock.patch.object(no.Node, 'get')
    def test_do_replace_nodes_node_not_active(self, mock_get_node, mock_load):
        cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=10)
        mock_load.return_value = cluster

        action = ca.ClusterAction(cluster.id, 'CLUSTER_ACTION', self.ctx)
        action.id = 'CLUSTER_ACTION_ID'
        action.inputs = {'ORIGIN_NODE': 'REPLACE_NODE'}
        action.outputs = {}

        origin_node = mock.Mock(id='ORIGIN_NODE', cluster_id='CLUSTER_ID',
                                ACTIVE='ACTIVE', status='ACTIVE')
        replace_node = mock.Mock(id='REPLACE_NODE', cluster_id='',
                                 ACTIVE='ACTIVE', status='ERROR')
        mock_get_node.side_effect = [origin_node, replace_node]

        # do it
        res_code, res_msg = action.do_replace_nodes()

        # assertions
        self.assertEqual(action.RES_ERROR, res_code)
        self.assertEqual("Node REPLACE_NODE is not in ACTIVE status.", res_msg)

    @mock.patch.object(ao.Action, 'update')
    @mock.patch.object(ab.Action, 'create')
    @mock.patch.object(no.Node, 'get')
    @mock.patch.object(dobj.Dependency, 'create')
    @mock.patch.object(dispatcher, 'start_action')
    @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
    def test_do_replace_failed_waiting(self, mock_wait, mock_start, mock_dep,
                                       mock_get_node, mock_action,
                                       mock_update, mock_load):
        cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=10)
        mock_load.return_value = cluster

        action = ca.ClusterAction(cluster.id, 'CLUSTER_ACTION', self.ctx)
        action.id = 'CLUSTER_ACTION_ID'
        action.inputs = {'O_NODE_1': 'R_NODE_1'}
        action.outputs = {}

        origin_node = mock.Mock(id='O_NODE_1', cluster_id='CLUSTER_ID',
                                ACTIVE='ACTIVE', status='ACTIVE')
        replace_node = mock.Mock(id='R_NODE_1', cluster_id='',
                                 ACTIVE='ACTIVE', status='ACTIVE')
        mock_get_node.side_effect = [origin_node, replace_node]
        mock_action.side_effect = ['NODE_LEAVE_1', 'NODE_JOIN_1']
        mock_wait.return_value = (action.RES_TIMEOUT, 'Timeout!')

        # do the action
        res_code, res_msg = action.do_replace_nodes()

        # assertions
        mock_action.assert_has_calls([
            mock.call(action.context, 'O_NODE_1', 'NODE_LEAVE',
                      name='node_leave_O_NODE_1',
                      cause='Derived Action'),
            mock.call(action.context, 'R_NODE_1', 'NODE_JOIN',
                      name='node_join_R_NODE_1',
                      cause='Derived Action',
                      inputs={'cluster_id': 'CLUSTER_ID'})])

        mock_dep.assert_has_calls([
            mock.call(action.context,
                      ['NODE_JOIN_1'],
                      'CLUSTER_ACTION_ID'),
            mock.call(action.context,
                      ['NODE_JOIN_1'],
                      'NODE_LEAVE_1')])

        mock_update.assert_has_calls([
            mock.call(action.context,
                      'NODE_JOIN_1',
                      {'status': 'READY'}),
            mock.call(action.context,
                      'NODE_LEAVE_1',
                      {'status': 'READY'})])

        self.assertEqual(action.RES_TIMEOUT, res_code)
        self.assertEqual('Timeout!', res_msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators
from tempest.lib import exceptions

from senlin.tests.tempest.api import base
from senlin.tests.tempest.common import utils


class TestReceiverCreateNegativeBadRequest(base.BaseSenlinAPITest):

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b55d204c-8ba2-43fd-bde4-d7d9d0e54c29')
    def test_receiver_create_receiver_data_not_specified(self):
        # receiver key is missing in request data
        params = {
            'receive': {
                'name': 'test-receiver',
                'cluster_id': 'CLUSTER_ID',
                'type': 'webhook',
                'action': 'CLUSTER_SCALE_IN',
                'params': {"count": 5}
            }
        }
        # Verify badrequest exception(400) is raised.
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_obj,
                          'receivers', params)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0e5cadcf-d9b7-4617-a83f-49557765f9bf')
    def test_receiver_create_unsupported_receiver_type(self):
        # Unsupported receiver type is specified
        params = {
            'receiver': {
                'name': 'test-receiver',
                'cluster_id': 'CLUSTER_ID',
                'type': 'bogus',
                'action': 'CLUSTER_SCALE_IN',
                'params': {"count": 5}
            }
        }
        # Verify badrequest exception(400) is raised.
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_obj,
                          'receivers', params)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e281873f-0dff-4be6-8795-f781efe7cc14')
    def test_receiver_create_target_cluster_not_found(self):
        # Target cluster cannot be found
        params = {
            'receiver': {
                'name': 'test-receiver',
                'cluster_id': 'e281873f-0dff-4be6-8795-f781efe7cc14',
                'type': 'webhook',
                'action': 'CLUSTER_SCALE_IN',
                'params': {"count": 5}
            }
        }
        # Verify badrequest exception(400) is raised.
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_obj,
                          'receivers', params)


class TestReceiverCreateNegativeInvalidAction(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestReceiverCreateNegativeInvalidAction, self).setUp()
        self.profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, self.profile_id)
        self.cluster_id = utils.create_a_cluster(self, self.profile_id)
        self.addCleanup(utils.delete_a_cluster, self, self.cluster_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('80e16a56-7d56-4038-b5f4-6d1ccd76c3f2')
    def test_receiver_create_invalid_action(self):
        # Target action type is illegal
        params = {
            'receiver': {
                'name': 'test-receiver',
                'cluster_id': self.cluster_id,
                'type': 'webhook',
                'action': 'ILLEGAL_ACTION',
                'params': {"count": 5}
            }
        }
        # Verify badrequest exception(400) is raised.
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_obj,
                          'receivers', params)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators
from tempest.lib import exceptions

from senlin.tests.tempest.api import base
from senlin.tests.tempest.common import utils


class TestPolicyDeleteNegativeConflict(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestPolicyDeleteNegativeConflict, self).setUp()
        profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, profile_id)

        cluster_id = utils.create_a_cluster(self, profile_id)
        self.addCleanup(utils.delete_a_cluster, self, cluster_id)

        self.policy_id = utils.create_a_policy(self)
        self.addCleanup(utils.delete_a_policy, self, self.policy_id)

        utils.cluster_attach_policy(self, cluster_id, self.policy_id)
        self.addCleanup(utils.cluster_detach_policy, self, cluster_id,
                        self.policy_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b8b8fca8-962f-4cad-bfca-76683df7b617')
    def test_policy_delete_conflict(self):
        # Verify conflict exception(409) is raised.
        ex = self.assertRaises(exceptions.Conflict,
                               self.client.delete_obj,
                               'policies', self.policy_id)

        message = ex.resp_body['error']['message']
        self.assertEqual(
            "The policy '%s' cannot be deleted: still attached to some "
            "clusters." % self.policy_id, str(message))


class TestPolicyDeleteNegativeNotFound(base.BaseSenlinAPITest):

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5591416f-4646-46c2-83b4-231e72aa4bfe')
    def test_policy_delete_not_found(self):
        # Verify notfound exception(404) is raised.
        ex = self.assertRaises(exceptions.NotFound,
                               self.client.delete_obj, 'policies',
                               '5591416f-4646-46c2-83b4-231e72aa4bfe')

        message = ex.resp_body['error']['message']
        self.assertEqual(
            "The policy '5591416f-4646-46c2-83b4-231e72aa4bfe' "
            "could not be found.", str(message))


class TestPolicyDeleteNegativeBadRequest(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestPolicyDeleteNegativeBadRequest, self).setUp()
        self.policy_id1 = utils.create_a_policy(self, name='p-01')
        self.addCleanup(utils.delete_a_policy, self, self.policy_id1)
        self.policy_id2 = utils.create_a_policy(self, name='p-01')
        self.addCleanup(utils.delete_a_policy, self, self.policy_id2)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d6f35043-2db5-49ff-8bc4-ba14a652f748')
    def test_policy_delete_multiple_choice(self):
        # Verify badrequest exception(400) is raised.
        ex = self.assertRaises(exceptions.BadRequest,
                               self.client.delete_obj,
                               'policies', 'p-01')

        message = ex.resp_body['error']['message']
        self.assertEqual(
            "Multiple results found matching the query criteria 'p-01'. "
            "Please be more specific.", str(message))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
from oslo_config import cfg
from oslo_log import log as logging
import testtools

from senlin.common import consts
from senlin.engine import event


class TestEvent(testtools.TestCase):

    def setUp(self):
        super(TestEvent, self).setUp()
        logging.register_options(cfg.CONF)

    @mock.patch('stevedore.named.NamedExtensionManager')
    def test_load_dispatcher(self, mock_mgr):

        class FakeDispatcher(object):
            values = {'a': 1, 'b': 2}

            def __iter__(self):
                return iter(self.values)

            def __getitem__(self, key):
                return self.values.get(key, '')

            def __contains__(self, name):
                return name in self.values

            def names(self):
                return self.values.keys()

        mock_mgr.return_value = FakeDispatcher()
        res = event.load_dispatcher()

        self.assertIsNone(res)
        mock_mgr.assert_called_once_with(
            namespace='senlin.dispatchers',
            names=cfg.CONF.event_dispatchers,
            invoke_on_load=True,
            propagate_map_exceptions=True)

    def test__event_data(self):
        entity = mock.Mock(id='ENTITY_ID')
        entity.name = 'FAKE_ENTITY'
        action = mock.Mock(id='ACTION_ID', action='ACTION', entity=entity)

        res = event._event_data(action)

        self.assertEqual({'name': 'FAKE_ENTITY', 'obj_id': 'ENTITY_I',
                          'action': 'ACTION', 'phase': None, 'reason': None,
                          'id': 'ACTION_I'},
                         res)

    def test__event_data_with_phase_reason(self):
        entity = mock.Mock(id='ENTITY_ID')
        entity.name = 'FAKE_ENTITY'
        action = mock.Mock(id='ACTION_ID', action='ACTION', entity=entity)

        res = event._event_data(action, phase='PHASE1', reason='REASON1')

        self.assertEqual({'name': 'FAKE_ENTITY', 'id': 'ACTION_I',
                          'action': 'ACTION', 'phase': 'PHASE1',
                          'obj_id': 'ENTITY_I', 'reason': 'REASON1'},
                         res)

    def test__dump(self):
        cfg.CONF.set_override('debug', True)
        saved_dispathers = event.dispatchers
        event.dispatchers = mock.Mock()
        action = mock.Mock(cause=consts.CAUSE_RPC)
        try:
            event._dump(logging.INFO, action, 'Phase1', 'Reason1', 'TS1')
            event.dispatchers.map_method.assert_called_once_with(
                'dump', logging.INFO, action,
                phase='Phase1', reason='Reason1', timestamp='TS1')
        finally:
            event.dispatchers = saved_dispathers

    def test__dump_without_timestamp(self):
        cfg.CONF.set_override('debug', True)
        saved_dispathers = event.dispatchers
        event.dispatchers = mock.Mock()
        action = mock.Mock(cause=consts.CAUSE_RPC)
        try:
            event._dump(logging.INFO, action, 'Phase1', 'Reason1', None)

            event.dispatchers.map_method.assert_called_once_with(
                'dump', logging.INFO, action,
                phase='Phase1', reason='Reason1', timestamp=mock.ANY)
        finally:
            event.dispatchers = saved_dispathers

    def test__dump_guarded(self):
        cfg.CONF.set_override('debug', False)
        cfg.CONF.set_override('priority', 'warning', group='dispatchers')
        saved_dispathers = event.dispatchers
        event.dispatchers = mock.Mock()
        action = mock.Mock(cause=consts.CAUSE_RPC)
        try:
            event._dump(logging.INFO, action, 'Phase1', 'Reason1', 'TS1')
            # (temporary)Remove map_method.call_count for coverage test
            # self.assertEqual(0, event.dispatchers.map_method.call_count)
        finally:
            event.dispatchers = saved_dispathers

    def test__dump_exclude_derived_actions_positive(self):
        cfg.CONF.set_override('exclude_derived_actions', True,
                              group='dispatchers')
        saved_dispathers = event.dispatchers
        event.dispatchers = mock.Mock()
        action = mock.Mock(cause=consts.CAUSE_DERIVED)
        try:
            event._dump(logging.INFO, action, 'Phase1', 'Reason1', 'TS1')

            self.assertEqual(0, event.dispatchers.map_method.call_count)
        finally:
            event.dispatchers = saved_dispathers

    def test__dump_exclude_derived_actions_negative(self):
        cfg.CONF.set_override('exclude_derived_actions', False,
                              group='dispatchers')
        saved_dispathers = event.dispatchers
        event.dispatchers = mock.Mock()
        action = mock.Mock(cause=consts.CAUSE_DERIVED)
        try:
            event._dump(logging.INFO, action, 'Phase1', 'Reason1', 'TS1')

            event.dispatchers.map_method.assert_called_once_with(
                'dump', logging.INFO, action,
                phase='Phase1', reason='Reason1', timestamp='TS1')
        finally:
            event.dispatchers = saved_dispathers

    def test__dump_with_exception(self):
        cfg.CONF.set_override('debug', True)
        saved_dispathers = event.dispatchers
        event.dispatchers = mock.Mock()
        event.dispatchers.map_method.side_effect = Exception('fab')
        action = mock.Mock(cause=consts.CAUSE_RPC)
        try:
            res = event._dump(logging.INFO, action, 'Phase1', 'Reason1', 'TS1')

            self.assertIsNone(res)  # exception logged only
            event.dispatchers.map_method.assert_called_once_with(
                'dump', logging.INFO, action,
                phase='Phase1', reason='Reason1', timestamp='TS1')
        finally:
            event.dispatchers = saved_dispathers


@mock.patch.object(event, '_dump')
class TestLogMethods(testtools.TestCase):

    def test_critical(self, mock_dump):
        entity = mock.Mock(id='1234567890')
        entity.name = 'fake_obj'
        action = mock.Mock(id='FAKE_ID', entity=entity, action='ACTION_NAME')

        res = event.critical(action, 'P1', 'R1', 'TS1')

        self.assertIsNone(res)
        mock_dump.assert_called_once_with(logging.CRITICAL, action,
                                          'P1', 'R1', 'TS1')

    def test_error(self, mock_dump):
        entity = mock.Mock(id='1234567890')
        entity.name = 'fake_obj'
        action = mock.Mock(id='FAKE_ID', entity=entity, action='ACTION_NAME')

        res = event.error(action, 'P1', 'R1', 'TS1')

        self.assertIsNone(res)
        mock_dump.assert_called_once_with(logging.ERROR, action,
                                          'P1', 'R1', 'TS1')

    def test_warning(self, mock_dump):
        entity = mock.Mock(id='1234567890')
        entity.name = 'fake_obj'
        action = mock.Mock(id='FAKE_ID', entity=entity, action='ACTION_NAME')

        res = event.warning(action, 'P1', 'R1', 'TS1')

        self.assertIsNone(res)
        mock_dump.assert_called_once_with(logging.WARNING, action,
                                          'P1', 'R1', 'TS1')

    def test_info(self, mock_dump):
        entity = mock.Mock(id='1234567890')
        entity.name = 'fake_obj'
        action = mock.Mock(id='FAKE_ID', entity=entity, action='ACTION_NAME')

        res = event.info(action, 'P1', 'R1', 'TS1')

        self.assertIsNone(res)
        mock_dump.assert_called_once_with(logging.INFO, action,
                                          'P1', 'R1', 'TS1')

    def test_debug(self, mock_dump):
        entity = mock.Mock(id='1234567890')
        entity.name = 'fake_obj'
        action = mock.Mock(id='FAKE_ID', entity=entity, action='ACTION_NAME')

        res = event.debug(action, 'P1', 'R1', 'TS1')

        self.assertIsNone(res)
        mock_dump.assert_called_once_with(logging.DEBUG, action,
                                          'P1', 'R1', 'TS1')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import timeutils
from oslo_utils import uuidutils
import testtools

from senlin.engine.actions import base as action_base
from senlin.engine import cluster
from senlin.engine import node
from senlin.events import base
from senlin.events import message as MSG
from senlin.objects import notification as nobj
from senlin.tests.unit.common import utils

CLUSTER_ID = '2c5139a6-24ba-4a6f-bd53-a268f61536de'


class TestMessageEvent(testtools.TestCase):

    def setUp(self):
        super(TestMessageEvent, self).setUp()
        self.ctx = utils.dummy_context()

    @mock.patch.object(nobj.NotificationBase, '_emit')
    def test__notify_cluster_action(self, mock_emit):
        cluster_id = uuidutils.generate_uuid()
        profile_id = uuidutils.generate_uuid()
        cluster_init = timeutils.utcnow(True)
        action_id = uuidutils.generate_uuid()
        cluster_params = {
            'id': cluster_id,
            'init_at': cluster_init,
            'min_size': 1,
            'max_size': 10,
            'timeout': 4,
            'status': 'ACTIVE',
            'status_reason': 'Good',
            'user': 'user1',
            'project': 'project1',
        }
        c1 = cluster.Cluster('fake_name', 5, profile_id, **cluster_params)
        action_params = {
            'id': action_id,
            'name': 'fake_name',
            'start_time': 1.23,
            'status': 'RUNNING',
            'status_reason': 'Good',
            'user': 'user1',
            'project': 'project1',
        }
        action = action_base.Action(cluster_id, 'CLUSTER_CREATE', self.ctx,
                                    **action_params)
        publisher_id = 'senlin-engine:%s' % cfg.CONF.host
        expected_payload = {
            'senlin_object.data': {
                'action': {
                    'senlin_object.data': {
                        'action': 'CLUSTER_CREATE',
                        'created_at': None,
                        'data': '{}',
                        'end_time': None,
                        'id': action_id,
                        'inputs': '{}',
                        'name': 'fake_name',
                        'outputs': '{}',
                        'project': self.ctx.project,
                        'start_time': 1.23,
                        'status': 'RUNNING',
                        'status_reason': 'Good',
                        'target': cluster_id,
                        'timeout': 3600,
                        'user': self.ctx.user,
                    },
                    'senlin_object.name': 'ActionPayload',
                    'senlin_object.namespace': 'senlin',
                    'senlin_object.version': '1.0'
                },
                'cluster': {
                    'senlin_object.data': {
                        'created_at': None,
                        'data': '{}',
                        'dependents': '{}',
                        'desired_capacity': 5,
                        'domain': '',
                        'id': cluster_id,
                        'init_at': mock.ANY,
                        'max_size': 10,
                        'metadata': '{}',
                        'min_size': 1,
                        'name': 'fake_name',
                        'profile_id': profile_id,
                        'project': u'project1',
                        'status': u'ACTIVE',
                        'status_reason': u'Good',
                        'timeout': 4,
                        'updated_at': None,
                        'user': u'user1'
                    },
                    'senlin_object.name': 'ClusterPayload',
                    'senlin_object.namespace': 'senlin',
                    'senlin_object.version': '1.0'
                },
                'exception': None
            },
            'senlin_object.name': 'ClusterActionPayload',
            'senlin_object.namespace': 'senlin',
            'senlin_object.version': '1.0'
        }

        res = MSG.MessageEvent._notify_cluster_action(
            self.ctx, logging.INFO, c1, action, phase='start')

        self.assertIsNone(res)

        mock_emit.assert_called_once_with(
            self.ctx, 'cluster.create.start', publisher_id, mock.ANY)
        payload = mock_emit.call_args[0][3]
        self.assertEqual(expected_payload, payload)

    @mock.patch.object(nobj.NotificationBase, '_emit')
    def test__notify_node_action(self, mock_emit):
        node_id = uuidutils.generate_uuid()
        profile_id = uuidutils.generate_uuid()
        node_init = timeutils.utcnow(True)
        action_id = uuidutils.generate_uuid()
        node_params = {
            'id': node_id,
            'cluster_id': '',
            'index': -1,
            'init_at': node_init,
            'status': 'ACTIVE',
            'status_reason': 'Good',
            'user': 'user1',
            'project': 'project1',
        }
        n1 = node.Node('fake_name', profile_id, **node_params)
        action_params = {
            'id': action_id,
            'name': 'fake_name',
            'start_time': 1.23,
            'status': 'RUNNING',
            'status_reason': 'Good',
        }
        action = action_base.Action(node_id, 'NODE_CREATE', self.ctx,
                                    **action_params)
        publisher_id = 'senlin-engine:%s' % cfg.CONF.host
        expected_payload = {
            'senlin_object.data': {
                'action': {
                    'senlin_object.data': {
                        'action': 'NODE_CREATE',
                        'created_at': None,
                        'data': '{}',
                        'end_time': None,
                        'id': action_id,
                        'inputs': '{}',
                        'name': 'fake_name',
                        'outputs': '{}',
                        'project': self.ctx.project,
                        'start_time': 1.23,
                        'status': 'RUNNING',
                        'status_reason': 'Good',
                        'target': node_id,
                        'timeout': 3600,
                        'user': self.ctx.user,
                    },
                    'senlin_object.name': 'ActionPayload',
                    'senlin_object.namespace': 'senlin',
                    'senlin_object.version': '1.0'
                },
                'node': {
                    'senlin_object.data': {
                        'cluster_id': '',
                        'created_at': None,
                        'data': '{}',
                        'dependents': '{}',
                        'domain': '',
                        'id': node_id,
                        'index': -1,
                        'init_at': mock.ANY,
                        'metadata': '{}',
                        'name': 'fake_name',
                        'physical_id': None,
                        'profile_id': profile_id,
                        'project': 'project1',
                        'role': '',
                        'status': 'ACTIVE',
                        'status_reason': 'Good',
                        'updated_at': None,
                        'user': 'user1',
                    },
                    'senlin_object.name': 'NodePayload',
                    'senlin_object.namespace': 'senlin',
                    'senlin_object.version': '1.0'
                },
                'exception': None
            },
            'senlin_object.name': 'NodeActionPayload',
            'senlin_object.namespace': 'senlin',
            'senlin_object.version': '1.0'
        }

        res = MSG.MessageEvent._notify_node_action(
            self.ctx, logging.INFO, n1, action, phase='start')

        self.assertIsNone(res)

        mock_emit.assert_called_once_with(
            self.ctx, 'node.create.start', publisher_id, mock.ANY)
        payload = mock_emit.call_args[0][3]
        self.assertEqual(expected_payload, payload)

    @mock.patch.object(MSG.MessageEvent, '_notify_cluster_action')
    @mock.patch.object(base.EventBackend, '_check_entity')
    def test_dump_cluster_action_event(self, mock_check, mock_notify):
        mock_check.return_value = 'CLUSTER'
        entity = mock.Mock()
        action = mock.Mock(context=self.ctx, entity=entity)

        res = MSG.MessageEvent.dump(logging.INFO, action)

        self.assertIsNone(res)
        mock_check.assert_called_once_with(entity)
        mock_notify.assert_called_once_with(self.ctx, logging.INFO, entity,
                                            action)

    @mock.patch.object(MSG.MessageEvent, '_notify_cluster_action')
    @mock.patch.object(base.EventBackend, '_check_entity')
    def test_dump_cluster_action_event_warn(self, mock_check, mock_notify):
        mock_check.return_value = 'CLUSTER'
        entity = mock.Mock()
        action = mock.Mock(context=self.ctx, entity=entity)

        res = MSG.MessageEvent.dump(logging.WARNING, action)

        self.assertIsNone(res)
        mock_check.assert_called_once_with(entity)
        mock_notify.assert_called_once_with(self.ctx, logging.WARNING,
                                            entity, action)

    @mock.patch.object(MSG.MessageEvent, '_notify_node_action')
    @mock.patch.object(base.EventBackend, '_check_entity')
    def test_dump_node_action_event(self, mock_check, mock_notify):
        mock_check.return_value = 'NODE'
        entity = mock.Mock()
        action = mock.Mock(context=self.ctx, entity=entity)

        res = MSG.MessageEvent.dump(logging.INFO, action)

        self.assertIsNone(res)
        mock_check.assert_called_once_with(entity)
        mock_notify.assert_called_once_with(self.ctx, logging.INFO, entity,
                                            action)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

import mock
from oslo_serialization import jsonutils
from oslo_utils import uuidutils
import six
from webob import exc

from senlin.api.common import util
from senlin.api.middleware import fault
from senlin.api.openstack.v1 import clusters
from senlin.common import exception as senlin_exc
from senlin.common import policy
from senlin.objects.requests import clusters as vorc
from senlin.rpc import client as rpc_client
from senlin.tests.unit.api import shared
from senlin.tests.unit.common import base
from senlin.tests.unit.common import utils


@mock.patch.object(policy, 'enforce')
class ClusterControllerTest(shared.ControllerTest, base.SenlinTestCase):
    """Test case for the cluster controller."""

    def setUp(self):
        super(ClusterControllerTest, self).setUp()

        class DummyConfig(object):
            bind_port = 8778

        cfgopts = DummyConfig()
        self.controller = clusters.ClusterController(options=cfgopts)
        self.context = utils.dummy_context()

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_index(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)
        req = self._get('/clusters')

        engine_resp = [{'foo': 'bar'}]
        mock_call.return_value = engine_resp
        obj = vorc.ClusterListRequest()
        mock_parse.return_value = obj

        result = self.controller.index(req)

        expected = {u'clusters': engine_resp}
        self.assertEqual(expected, result)

        mock_parse.assert_called_once_with('ClusterListRequest', req,
                                           {'project_safe': True})
        mock_call.assert_called_once_with(req.context, 'cluster_list', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_index_with_params(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)
        fake_id = uuidutils.generate_uuid()
        params = {
            'name': 'name1',
            'status': 'ACTIVE',
            'limit': '3',
            'marker': fake_id,
            'sort': 'name:asc',
            'global_project': 'True',
        }
        req = self._get('/clusters', params=params)
        obj = vorc.ClusterListRequest()
        mock_parse.return_value = obj
        engine_resp = [{'foo': 'bar'}]
        mock_call.return_value = engine_resp

        result = self.controller.index(req)

        expected = {u'clusters': engine_resp}
        self.assertEqual(expected, result)
        mock_parse.assert_called_once_with(
            'ClusterListRequest', req,
            {
                'name': ['name1'],
                'status': ['ACTIVE'],
                'limit': '3',
                'marker': fake_id,
                'sort': 'name:asc',
                'project_safe': False
            })

        mock_call.assert_called_once_with(req.context, 'cluster_list', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_index_failed_with_exception(self, mock_call, mock_parse,
                                         mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)
        req = self._get('/clusters', params={})
        mock_parse.side_effect = exc.HTTPBadRequest("Boom")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.index,
                               req)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            "ClusterListRequest", req, {'project_safe': True})
        self.assertEqual(0, mock_call.call_count)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_index_failed_engine_error(self, mock_call, mock_parse,
                                       mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', True)
        params = {'global_project': True}
        req = self._get('/clusters', params=params)
        obj = mock.Mock()
        mock_parse.return_value = obj
        error = senlin_exc.Forbidden()
        mock_call.side_effect = shared.to_remote_error(error)

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.index,
                                              req)

        self.assertEqual(403, resp.json['code'])
        self.assertEqual('Forbidden', resp.json['error']['type'])
        mock_parse.assert_called_once_with(
            "ClusterListRequest", mock.ANY, {'project_safe': False})
        mock_call.assert_called_once_with(req.context, 'cluster_list', obj)

    def test_index_error_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'index', False)
        req = self._get('/clusters')

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.index,
                                              req)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_create(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'create', True)
        body = {
            'cluster': {
                'name': 'test_cluster',
                'desired_capacity': 0,
                'profile_id': 'xxxx-yyyy',
                'min_size': 0,
                'max_size': 0,
                'metadata': {},
                'timeout': None,
            }
        }
        req = self._post('/clusters', jsonutils.dumps(body))
        engine_response = {
            'id': 'FAKE_ID',
            'name': 'test_cluster',
            'desired_capacity': 0,
            'profile_id': 'xxxx-yyyy',
            'min_size': 0,
            'max_size': 0,
            'metadata': {},
            'timeout': 60,
            'action': 'fake_action'
        }
        mock_call.return_value = engine_response
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller.create(req, body=body)

        self.assertEqual(engine_response, resp['cluster'])
        self.assertEqual('/actions/fake_action', resp['location'])
        mock_parse.assert_called_once_with(
            "ClusterCreateRequest", mock.ANY, body, 'cluster')
        mock_call.assert_called_once_with(req.context, 'cluster_create',
                                          obj.cluster)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_create_failed_request(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'create', True)
        body = {'foo': 'bar'}
        req = self._post('/clusters', jsonutils.dumps(body))
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.create,
                               req, body=body)

        self.assertEqual("Boom", six.text_type(ex))
        self.assertEqual(0, mock_call.call_count)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_create_failed_engine(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'create', True)
        body = {'foo': 'bar'}
        req = self._post('/clusters', jsonutils.dumps(body))
        obj = mock.Mock()
        mock_parse.return_value = obj
        error = senlin_exc.BadRequest(msg='bad')
        mock_call.side_effect = shared.to_remote_error(error)

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.create,
                                              req, body=body)

        self.assertEqual(400, resp.json['code'])
        self.assertEqual('BadRequest', resp.json['error']['type'])
        mock_parse.assert_called_once_with(
            "ClusterCreateRequest", mock.ANY, {'foo': 'bar'}, 'cluster')
        mock_call.assert_called_once_with(req.context, 'cluster_create',
                                          obj.cluster)

    def test_create_err_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'create', False)
        body = {
            'cluster': {
                'name': 'test_cluster',
                'profile_id': 'xxxx-yyyy',
            }
        }
        req = self._post('/clusters', jsonutils.dumps(body))

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.create,
                                              req, body=body)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_get(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'get', True)
        cid = 'cid'
        req = self._get('/clusters/%s' % cid)
        engine_resp = {'foo': 'bar'}
        mock_call.return_value = engine_resp
        obj = mock.Mock()
        mock_parse.return_value = obj

        response = self.controller.get(req, cluster_id=cid)

        self.assertEqual({'cluster': {'foo': 'bar'}}, response)
        mock_parse.assert_called_once_with(
            "ClusterGetRequest", req, {'identity': 'cid'})
        mock_call.assert_called_once_with(req.context, 'cluster_get', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_get_failed_request(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'get', True)
        cid = 'FAKE_ID'
        req = self._get('/clusters/%s' % cid)
        mock_parse.side_effect = exc.HTTPBadRequest("Boom")
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.get,
                               req, cluster_id=cid)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            "ClusterGetRequest", req, {'identity': cid})
        self.assertEqual(0, mock_call.call_count)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_get_failed_engine(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'get', True)
        cid = 'non-existent-cluster'
        req = self._get('/clusters/%s' % cid)

        error = senlin_exc.ResourceNotFound(type='cluster', id=cid)
        mock_call.side_effect = shared.to_remote_error(error)

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.get,
                                              req, cluster_id=cid)

        self.assertEqual(404, resp.json['code'])
        self.assertEqual('ResourceNotFound', resp.json['error']['type'])
        mock_parse.assert_called_once_with(
            "ClusterGetRequest", mock.ANY, {'identity': cid})

    def test_get_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'get', False)
        cid = 'cid'
        req = self._get('/clusters/%s' % cid)

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.get,
                                              req, cluster_id=cid)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_update(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', True)
        cid = 'aaaa-bbbb-cccc'
        body = {'cluster': {'foo': 'bar'}}
        engine_resp = {
            'id': cid,
            'action': 'fake_action',
        }
        req = self._patch('/clusters/%s' % cid, jsonutils.dumps(body))
        mock_call.return_value = engine_resp
        obj = mock.Mock()
        mock_parse.return_value = obj

        res = self.controller.update(req, cluster_id=cid, body=body)

        self.assertEqual(
            {'cluster': {'id': cid}, 'location': '/actions/fake_action'},
            res)
        mock_parse.assert_called_once_with(
            "ClusterUpdateRequest", req,
            {'identity': 'aaaa-bbbb-cccc', 'foo': 'bar'})
        mock_call.assert_called_once_with(req.context, 'cluster_update', obj)

    def test_update_missing_cluster_key(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', True)
        cid = 'aaaa-bbbb-cccc'
        body = {'profile_id': 'xxxx-yyyy-zzzz'}
        req = self._patch('/clusters/%s' % cid, jsonutils.dumps(body))
        mock_call = self.patchobject(rpc_client.EngineClient, 'call')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.update,
                               req, cluster_id=cid, body=body)

        self.assertIn("Malformed request data, missing 'cluster' key "
                      "in request body.", six.text_type(ex))
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_update_failed_request(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', True)
        cid = 'aaaa-bbbb-cccc'
        body = {'cluster': {'name': 'foo bar'}}
        req = self._patch('/clusters/%s' % cid, jsonutils.dumps(body))
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.update,
                               req, cluster_id=cid, body=body)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            "ClusterUpdateRequest", req,
            {'identity': 'aaaa-bbbb-cccc', 'name': 'foo bar'})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_update_engine_error(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', True)
        cid = 'non-existent-cluster'
        body = {'cluster': {'profile_id': 'xxxx-yyyy-zzzz'}}
        req = self._patch('/clusters/%s' % cid, jsonutils.dumps(body))
        obj = mock.Mock()
        mock_parse.return_value = obj
        error = senlin_exc.ResourceNotFound(type='cluster', id=cid)
        mock_call.side_effect = shared.to_remote_error(error)

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.update,
                                              req, cluster_id=cid, body=body)

        self.assertEqual(404, resp.json['code'])
        self.assertEqual('ResourceNotFound', resp.json['error']['type'])
        mock_parse.assert_called_once_with(
            "ClusterUpdateRequest", mock.ANY,
            {'identity': cid, 'profile_id': 'xxxx-yyyy-zzzz'})
        mock_call.assert_called_once_with(req.context, 'cluster_update', obj)

    def test_update_err_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'update', False)
        cid = 'aaaa-bbbb-cccc'
        body = {'cluster': {'profile_id': 'xxxx-yyyy-zzzz'}}

        req = self._patch('/clusters/%(cluster_id)s' % {'cluster_id': cid},
                          jsonutils.dumps(body))

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.update,
                                              req, cluster_id=cid, body=body)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_add_nodes(self, mock_call, mock_parse, mock_enforce):
        req = mock.Mock()
        cid = 'FAKE_ID'
        data = dict(nodes=['NODE1'])
        mock_call.return_value = {'action': 'action-id'}
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller._do_add_nodes(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        mock_parse.assert_called_once_with(
            'ClusterAddNodesRequest',
            req,
            {'identity': cid, 'nodes': data['nodes']}
        )
        mock_call.assert_called_once_with(
            req.context, 'cluster_add_nodes', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_add_nodes_failed_request(self, mock_call,
                                          mock_parse, _ignore):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(nodes=['NODE2'])
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_add_nodes,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterAddNodesRequest',
            req,
            {'identity': cid, 'nodes': data['nodes']}
        )
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__add_nodes_failed_engine(self, mock_call, mock_parse, _ignore):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(nodes=['NODE3'])
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_add_nodes,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterAddNodesRequest',
            req,
            {'identity': cid, 'nodes': data['nodes']}
        )
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_add_nodes', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_del_nodes(self, mock_call, mock_parse, _ignore):
        req = mock.Mock()
        cid = 'FAKE_ID'
        data = dict(nodes=['NODE4'], destroy=False)
        mock_call.return_value = {'action': 'action-id'}
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller._do_del_nodes(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        mock_parse.assert_called_once_with(
            'ClusterDelNodesRequest', req, {'identity': cid,
                                            'nodes': data['nodes'],
                                            'destroy_after_deletion': False})
        mock_call.assert_called_once_with(
            req.context, 'cluster_del_nodes', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_del_nodes_failed_request(self, mock_call,
                                          mock_parse, _ignore):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(nodes=['NODE5'], destroy=False)
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_del_nodes,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterDelNodesRequest', req, {'identity': cid,
                                            'nodes': data['nodes'],
                                            'destroy_after_deletion': False})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_del_nodes_failed_engine(self, mock_call, mock_parse, _ignore):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(nodes=['NODE6'], destroy=False)
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_del_nodes,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterDelNodesRequest', req, {'identity': cid,
                                            'nodes': data['nodes'],
                                            'destroy_after_deletion': False})
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_del_nodes', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_replace_nodes(self, mock_call, mock_parse, _ignore):
        req = mock.Mock()
        cid = 'FAKE_ID'
        data = dict(nodes={'OLD': 'NEW'})
        mock_call.return_value = {'action': 'action-id'}
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller._do_replace_nodes(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        mock_parse.assert_called_once_with(
            'ClusterReplaceNodesRequest',
            req,
            {'identity': cid, 'nodes': data['nodes']}
        )
        mock_call.assert_called_once_with(
            req.context, 'cluster_replace_nodes', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_replace_nodes_none(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(nodes=None)
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_replace_nodes,
                               req, cid, data)

        self.assertEqual("The data provided is not a map", six.text_type(ex))
        self.assertEqual(0, mock_parse.call_count)
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_replace_nodes_failed_request(self, mock_call,
                                              mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(nodes={'OLD': 'NEW'})
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_replace_nodes,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterReplaceNodesRequest',
            req,
            {'identity': cid, 'nodes': data['nodes']}
        )
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_replace_nodes_failed_engine(self, mock_call,
                                             mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(nodes={'OLD': 'NEW'})
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_replace_nodes,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterReplaceNodesRequest',
            req,
            {'identity': cid, 'nodes': data['nodes']}
        )
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_replace_nodes', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def _test_do_resize_with_type(self, adj_type, mock_call, mock_parse):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {
            'adjustment_type': adj_type,
            'number': 1,
            'min_size': 0,
            'max_size': 10,
            'min_step': 1,
            'strict': True
        }
        mock_call.return_value = {'action': 'action-id'}
        # We are using a real object for testing
        obj = vorc.ClusterResizeRequest(identity=cid, **data)
        mock_parse.return_value = obj

        resp = self.controller._do_resize(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        params = copy.deepcopy(data)
        if params['adjustment_type'] != 'CHANGE_IN_PERCENTAGE':
            params.pop('min_step')
        params['identity'] = cid
        mock_parse.assert_called_once_with(
            'ClusterResizeRequest', req, params)
        mock_call.assert_called_once_with(req.context, 'cluster_resize', obj)

    def test__do_resize_exact_capacity(self, mock_enforce):
        self._test_do_resize_with_type('EXACT_CAPACITY')

    def test__do_resize_with_change_capacity(self, mock_enforce):
        self._test_do_resize_with_type('CHANGE_IN_CAPACITY')

    def test__do_resize_with_change_percentage(self, mock_enforce):
        self._test_do_resize_with_type('CHANGE_IN_PERCENTAGE')

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_resize_failed_request(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'adjustment_type': 'EXACT_CAPACITY', 'number': 10}
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_resize,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterResizeRequest', req,
            {
                'identity': cid,
                'adjustment_type': 'EXACT_CAPACITY',
                'number': 10
            })
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_resize_missing_number(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'adjustment_type': 'EXACT_CAPACITY'}
        obj = vorc.ClusterResizeRequest(identity=cid, **data)
        mock_parse.return_value = obj
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_resize,
                               req, cid, data)

        self.assertEqual('Missing number value for size adjustment.',
                         six.text_type(ex))
        self.assertEqual(0, mock_call.call_count)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_resize_missing_type(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'number': 2}
        obj = vorc.ClusterResizeRequest(identity=cid, **data)
        mock_parse.return_value = obj

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_resize,
                               req, cid, data)

        self.assertEqual("Missing adjustment_type value for size adjustment.",
                         six.text_type(ex))
        self.assertEqual(0, mock_call.call_count)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_resize_max_size_too_small(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'min_size': 2, 'max_size': 1}
        obj = vorc.ClusterResizeRequest(identity=cid, **data)
        mock_parse.return_value = obj

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_resize,
                               req, cid, data)

        self.assertEqual("The specified min_size (2) is greater than "
                         "the specified max_size (1).", six.text_type(ex))
        self.assertEqual(0, mock_call.call_count)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_resize_empty_params(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {}

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_resize,
                               req, cid, data)

        self.assertEqual("Not enough parameters to do resize action.",
                         six.text_type(ex))
        self.assertEqual(0, mock_call.call_count)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_resize_failed_engine(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'max_size': 200}
        obj = vorc.ClusterResizeRequest(identity=cid, **data)
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_resize,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterResizeRequest', req, {'identity': cid, 'max_size': 200})
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_resize', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_scale_out(self, mock_call, mock_parse, _ignore):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(count=1)
        mock_call.return_value = {'action': 'action-id'}
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller._do_scale_out(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        mock_parse.assert_called_once_with(
            'ClusterScaleOutRequest',
            req,
            {'identity': cid, 'count': data['count']}
        )
        mock_call.assert_called_once_with(
            req.context, 'cluster_scale_out', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_scale_out_failed_request(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(count=2)
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_scale_out,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterScaleOutRequest',
            req,
            {'identity': cid, 'count': data['count']}
        )
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_scale_out_failed_engine(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(count=3)
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_scale_out,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterScaleOutRequest',
            req,
            {'identity': cid, 'count': data['count']}
        )
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_scale_out', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_scale_in(self, mock_call, mock_parse, _ignore):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(count=4)
        mock_call.return_value = {'action': 'action-id'}
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller._do_scale_in(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        mock_parse.assert_called_once_with(
            'ClusterScaleInRequest',
            req,
            {'identity': cid, 'count': data['count']}
        )
        mock_call.assert_called_once_with(
            req.context, 'cluster_scale_in', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_scale_in_failed_request(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(count=5)
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_scale_in,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterScaleInRequest',
            req,
            {'identity': cid, 'count': data['count']}
        )
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_scale_in_failed_engine(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = dict(count=6)
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_scale_in,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterScaleInRequest',
            req,
            {'identity': cid, 'count': data['count']}
        )
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_scale_in', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_policy_attach(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'policy_id': 'xxxx-yyyy'}
        mock_call.return_value = {'action': 'action-id'}
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller._do_policy_attach(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        mock_parse.assert_called_once_with(
            'ClusterAttachPolicyRequest', req,
            {'identity': cid, 'policy_id': 'xxxx-yyyy'})
        mock_call.assert_called_once_with(
            req.context, 'cluster_policy_attach', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_policy_attach_failed_request(self, mock_call, mock_parse, _i):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'policy_id': 'xxxx-yyyy'}
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_policy_attach,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterAttachPolicyRequest', req,
            {'identity': cid, 'policy_id': 'xxxx-yyyy'})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_policy_attach_failed_engine(self, mock_call, mock_parse, _i):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'policy_id': 'xxxx-yyyy'}
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_policy_attach,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterAttachPolicyRequest', req,
            {'identity': cid, 'policy_id': 'xxxx-yyyy'})
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_policy_attach', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_policy_detach(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'policy_id': 'xxxx-yyyy'}
        mock_call.return_value = {'action': 'action-id'}
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller._do_policy_detach(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        mock_parse.assert_called_once_with(
            'ClusterDetachPolicyRequest', req,
            {'identity': cid, 'policy_id': 'xxxx-yyyy'})
        mock_call.assert_called_once_with(
            req.context, 'cluster_policy_detach', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_policy_detach_failed_request(self, mock_call, mock_parse, _i):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'policy_id': 'xxxx-yyyy'}
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_policy_detach,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterDetachPolicyRequest', req,
            {'identity': cid, 'policy_id': 'xxxx-yyyy'})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_policy_detach_failed_engine(self, mock_call, mock_parse, _i):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'policy_id': 'xxxx-yyyy'}
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_policy_detach,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterDetachPolicyRequest', req,
            {'identity': cid, 'policy_id': 'xxxx-yyyy'})
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_policy_detach', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_policy_update(self, mock_call, mock_parse, _ign):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'policy_id': 'xxxx-yyyy'}
        mock_call.return_value = {'action': 'action-id'}
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller._do_policy_update(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        mock_parse.assert_called_once_with(
            'ClusterUpdatePolicyRequest', req,
            {'identity': cid, 'policy_id': 'xxxx-yyyy'})
        mock_call.assert_called_once_with(
            req.context, 'cluster_policy_update', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_policy_update_failed_request(self, mock_call, mock_parse, _i):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'policy_id': 'xxxx-yyyy'}
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_policy_update,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterUpdatePolicyRequest', req,
            {'identity': cid, 'policy_id': 'xxxx-yyyy'})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_policy_update_failed_engine(self, mock_call, mock_parse, _i):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'policy_id': 'xxxx-yyyy'}
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_policy_update,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterUpdatePolicyRequest', req,
            {'identity': cid, 'policy_id': 'xxxx-yyyy'})
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_policy_update', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_check(self, mock_call, mock_parse, _ignore):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'op': 'value'}
        obj = mock.Mock()
        mock_parse.return_value = obj
        eng_resp = {'action': 'action-id'}
        mock_call.return_value = eng_resp

        resp = self.controller._do_check(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        mock_parse.assert_called_once_with(
            'ClusterCheckRequest', req,
            {'identity': cid, 'params': {'op': 'value'}})
        mock_call.assert_called_once_with(req.context, 'cluster_check', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_check_failed_request(self, mock_call, mock_parse, _ign):
        cid = 'fake-cluster'
        data = {}
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_check,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterCheckRequest', req, {'identity': cid, 'params': {}})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_check_failed_engine(self, mock_call, mock_parse, _i):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {}
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_check,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterCheckRequest', req, {'identity': cid, 'params': {}})
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_check', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_recover(self, mock_call, mock_parse, _ignore):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {'op': 'value'}
        obj = mock.Mock()
        mock_parse.return_value = obj
        eng_resp = {'action': 'action-id'}
        mock_call.return_value = eng_resp

        resp = self.controller._do_recover(req, cid, data)

        self.assertEqual({'action': 'action-id'}, resp)
        mock_parse.assert_called_once_with(
            'ClusterRecoverRequest', req,
            {'identity': cid, 'params': {'op': 'value'}})
        mock_call.assert_called_once_with(req.context, 'cluster_recover', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_recover_failed_request(self, mock_call, mock_parse, _ign):
        cid = 'fake-cluster'
        data = {}
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller._do_recover,
                               req, cid, data)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterRecoverRequest', req, {'identity': cid, 'params': {}})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test__do_recover_failed_engine(self, mock_call, mock_parse, _i):
        req = mock.Mock()
        cid = 'aaaa-bbbb-cccc'
        data = {}
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller._do_recover,
                               req, cid, data)

        mock_parse.assert_called_once_with(
            'ClusterRecoverRequest', req, {'identity': cid, 'params': {}})
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_recover', obj)

    def test_cluster_action_missing_action(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'action', True)
        cid = 'aaaa-bbbb-cccc'
        body = {}
        req = self._post('/clusters/%s/actions' % cid, jsonutils.dumps(body))

        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.action,
                               req, cluster_id=cid, body=body)
        self.assertEqual('No action specified', six.text_type(ex))
        self.assertFalse(mock_call.called)

    def test_cluster_action_multiple_actions(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'action', True)
        cid = 'aaaa-bbbb-cccc'
        body = {'action_1': {}, 'action_2': {}}
        req = self._post('/clusters/%s/actions' % cid, jsonutils.dumps(body))

        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.action,
                               req, cluster_id=cid, body=body)
        self.assertEqual('Multiple actions specified', six.text_type(ex))
        self.assertFalse(mock_call.called)

    def test_cluster_action_unsupported_action(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'action', True)
        cid = 'aaaa-bbbb-cccc'
        body = {'fly': None}
        req = self._post('/clusters/%s/actions' % cid, jsonutils.dumps(body))

        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.action,
                               req, cluster_id=cid, body=body)
        self.assertEqual("Unrecognized action 'fly' specified",
                         six.text_type(ex))
        self.assertFalse(mock_call.called)

    def test_cluster_action_err_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'action', False)
        cid = 'aaaa-bbbb-cccc'
        body = {'someaction': {'param': 'value'}}

        req = self._post('/clusters/%s/actions' % cid, jsonutils.dumps(body))

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.action,
                                              req, cluster_id=cid, body=body)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    def test_cluster_action_data_not_map(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'action', True)
        cid = 'aaaa-bbbb-cccc'
        body = {'resize': ['param1', 'param2']}

        req = self._post('/clusters/%s/actions' % cid, jsonutils.dumps(body))

        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.action,
                               req, cluster_id=cid, body=body)
        self.assertEqual('The data provided is not a map', six.text_type(ex))
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_collect(self, mock_call, mock_parse, mock_enforce):
        req = mock.Mock(context=self.context)
        cid = 'aaaa-bbbb-cccc'
        path = 'foo.bar'
        eng_resp = {'cluster_attributes': [{'key': 'value'}]}
        mock_call.return_value = eng_resp
        obj = vorc.ClusterResizeRequest(identity=cid, path=path)
        mock_parse.return_value = obj

        resp = self.controller.collect(req, cluster_id=cid, path=path)

        self.assertEqual(eng_resp, resp)
        mock_call.assert_called_once_with(req.context, 'cluster_collect', obj)

    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_collect_version_mismatch(self, mock_call, mock_enforce):
        # NOTE: we skip the mock_enforce setup below because api version check
        #       comes before the policy enforcement and the check fails in
        #       this test case.
        cid = 'aaaa-bbbb-cccc'
        path = 'foo.bar'
        req = self._get('/clusters/%(cid)s/attrs/%(path)s' %
                        {'cid': cid, 'path': path}, version='1.1')

        ex = self.assertRaises(senlin_exc.MethodVersionNotFound,
                               self.controller.collect,
                               req, cluster_id=cid, path=path)

        self.assertEqual(0, mock_call.call_count)
        self.assertEqual("API version '1.1' is not supported on this method.",
                         six.text_type(ex))

    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_collect_path_not_provided(self, mock_call, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'collect', True)
        req = mock.Mock(context=self.context)
        cid = 'aaaa-bbbb-cccc'
        path = '    '
        req = self._get('/clusters/%(cid)s/attrs/%(path)s' %
                        {'cid': cid, 'path': path}, version='1.2')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.collect,
                               req, cluster_id=cid, path=path)

        self.assertEqual('Required path attribute is missing.',
                         six.text_type(ex))
        self.assertEqual(0, mock_call.call_count)

    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_collect_path_is_none(self, mock_call, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'collect', True)
        req = mock.Mock(context=self.context)
        cid = 'aaaa-bbbb-cccc'
        path = 'None'
        req = self._get('/clusters/%(cid)s/attrs/%(path)s' %
                        {'cid': cid, 'path': path}, version='1.2')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.collect,
                               req, cluster_id=cid, path=path)

        self.assertEqual('Required path attribute is missing.',
                         six.text_type(ex))
        self.assertEqual(0, mock_call.call_count)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_collect_failed_request(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'collect', True)
        req = mock.Mock(context=self.context)
        cid = 'aaaa-bbbb-cccc'
        path = 'foo.bar'
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.collect,
                               req, cluster_id=cid, path=path)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterCollectRequest', req, {'identity': cid, 'path': path})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_collect_failed_engine(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'collect', True)
        req = mock.Mock(context=self.context)
        cid = 'aaaa-bbbb-cccc'
        path = 'foo.bar'
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.side_effect = senlin_exc.BadRequest(msg='Boom')

        ex = self.assertRaises(senlin_exc.BadRequest,
                               self.controller.collect,
                               req, cluster_id=cid, path=path)

        mock_parse.assert_called_once_with(
            'ClusterCollectRequest', req, {'identity': cid, 'path': path})
        self.assertEqual("Boom.", six.text_type(ex))
        mock_call.assert_called_once_with(
            req.context, 'cluster_collect', obj)

    def test_collect_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'collect', False)
        cid = 'aaaa-bbbb-cccc'
        path = 'foo.bar'
        req = self._get('/clusters/%(cid)s/attrs/%(path)s' %
                        {'cid': cid, 'path': path}, version='1.2')
        mock_call = self.patchobject(rpc_client.EngineClient, 'call')

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.collect,
                                              req, cluster_id=cid, path=path)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))
        self.assertEqual(0, mock_call.call_count)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_operation(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'operation', True)
        cid = 'aaaa-bbbb-cccc'
        body = {
            'dance': {
                'params': {
                    'style': 'tango'
                },
                'filters': {
                    'role': 'slave'
                }
            }
        }
        req = self._post('/clusters/aaaa-bbbb-cccc/ops',
                         jsonutils.dumps(body), version='1.4')
        eng_resp = {'action': 'ACTION_ID'}
        mock_call.return_value = eng_resp
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = self.controller.operation(req, cluster_id=cid, body=body)

        self.assertEqual(eng_resp, resp)
        mock_call.assert_called_once_with(req.context, 'cluster_op', obj)

    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_operation_version_mismatch(self, mock_call, mock_enforce):
        cid = 'aaaa-bbbb-cccc'
        body = {'dance': {}}
        req = self._post('/clusters/aaaa-bbbb/ops', jsonutils.dumps(body),
                         version='1.1')

        ex = self.assertRaises(senlin_exc.MethodVersionNotFound,
                               self.controller.operation,
                               req, cluster_id=cid, body=body)

        self.assertEqual(0, mock_call.call_count)
        self.assertEqual("API version '1.1' is not supported on this method.",
                         six.text_type(ex))

    def test_operation_no_operations(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'operation', True)
        cid = 'aaaa-bbbb-cccc'
        body = {}
        req = self._post('/clusters/aaaa-bbbb-cccc/ops',
                         jsonutils.dumps(body), version='1.4')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.operation,
                               req, cluster_id=cid, body=body)

        self.assertEqual("No operation specified", six.text_type(ex))

    def test_operation_multi_operations(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'operation', True)
        cid = 'aaaa-bbbb-cccc'
        body = {'dance': {}, 'sing': {}}
        req = self._post('/clusters/aaaa-bbbb-cccc/ops',
                         jsonutils.dumps(body), version='1.4')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.operation,
                               req, cluster_id=cid, body=body)

        self.assertEqual("Multiple operations specified", six.text_type(ex))

    def test_cluster_operation_err_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'operation', False)
        body = {'someoperation': {}}
        req = self._post('/clusters/abc/ops', jsonutils.dumps(body),
                         version='1.4')

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.operation,
                                              req, cluster_id='abc', body=body)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_delete(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'delete', True)
        req = mock.Mock(context=self.context)
        req.params.get.return_value = 'false'
        cid = 'aaaa-bbbb-cccc'
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.return_value = {'action': 'FAKE_ID'}

        res = self.controller.delete(req, cluster_id=cid)

        result = {'location': '/actions/FAKE_ID'}
        self.assertEqual(result, res)
        mock_parse.assert_called_once_with(
            'ClusterDeleteRequest', req, {'identity': cid, 'force': False})

        mock_call.assert_called_with(req.context, 'cluster_delete', obj)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_delete_failed_request(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'delete', True)
        cid = 'fake-cluster'
        req = mock.Mock(context=self.context)
        req.params.get.return_value = 'false'
        mock_parse.side_effect = exc.HTTPBadRequest('Boom')

        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.controller.delete,
                               req, cluster_id=cid)

        self.assertEqual("Boom", six.text_type(ex))
        mock_parse.assert_called_once_with(
            'ClusterDeleteRequest', req, {'identity': cid, 'force': False})
        self.assertFalse(mock_call.called)

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_delete_failed_engine(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'delete', True)
        cid = 'aaaa-bbbb-cccc'
        req = self._delete('/clusters/%s' % cid, params={'force': 'false'})
        error = senlin_exc.ResourceNotFound(type='cluster', id=cid)
        mock_call.side_effect = shared.to_remote_error(error)
        obj = mock.Mock()
        mock_parse.return_value = obj

        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.delete,
                                              req, cluster_id=cid)

        self.assertEqual(404, resp.json['code'])
        self.assertEqual('ResourceNotFound', resp.json['error']['type'])

    def test_delete_err_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'delete', False)
        cid = 'aaaa-bbbb-cccc'
        req = self._delete('/clusters/%s' % cid)
        resp = shared.request_with_middleware(fault.FaultWrapper,
                                              self.controller.delete,
                                              req, cluster_id=cid)

        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', six.text_type(resp))

    @mock.patch.object(util, 'parse_request')
    @mock.patch.object(rpc_client.EngineClient, 'call')
    def test_delete_force(self, mock_call, mock_parse, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'delete', True)
        req = mock.Mock(context=self.context)
        req.params.get.return_value = 'true'
        cid = 'aaaa-bbbb-cccc'
        obj = mock.Mock()
        mock_parse.return_value = obj
        mock_call.return_value = {'action': 'FAKE_ID'}

        params = {'cluster_id': cid, 'body': {'force': True}}
        res = self.controller.delete(req, **params)

        result = {'location': '/actions/FAKE_ID'}
        self.assertEqual(result, res)
        mock_parse.assert_called_once_with(
            'ClusterDeleteRequest', req, {'identity': cid, 'force': True})

        mock_call.assert_called_with(req.context, 'cluster_delete', obj)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import six

from oslo_config import cfg

from senlin.common import consts
from senlin.objects.requests import receivers
from senlin.tests.unit.common import base as test_base

CONF = cfg.CONF
CONF.import_opt('default_action_timeout', 'senlin.common.config')


class TestReceiverCreate(test_base.SenlinTestCase):

    body = {
        'name': 'test-receiver',
        'type': 'message',
    }

    def test_receiver_create_request_body(self):
        sot = receivers.ReceiverCreateRequestBody(**self.body)
        self.assertEqual('test-receiver', sot.name)
        self.assertEqual('message', sot.type)

        sot.obj_set_defaults()

        self.assertEqual({}, sot.actor)
        self.assertEqual({}, sot.params)
        self.assertFalse(sot.obj_attr_is_set('action'))
        self.assertFalse(sot.obj_attr_is_set('cluster_id'))

    def test_receiver_create_request_body_full(self):
        body = copy.deepcopy(self.body)
        body['type'] = 'webhook'
        body['cluster_id'] = 'cluster-01'
        body['action'] = consts.CLUSTER_SCALE_OUT
        body['actor'] = {'user': 'user1', 'password': 'pass1'}
        body['params'] = {'count': '1'}
        sot = receivers.ReceiverCreateRequestBody(**body)
        self.assertEqual('test-receiver', sot.name)
        self.assertEqual('webhook', sot.type)
        self.assertEqual('cluster-01', sot.cluster_id)
        self.assertEqual(consts.CLUSTER_SCALE_OUT, sot.action)
        self.assertEqual({'user': 'user1', 'password': 'pass1'}, sot.actor)
        self.assertEqual({'count': '1'}, sot.params)

    def test_receiver_create_request_body_invalid_type(self):
        body = copy.deepcopy(self.body)
        body['type'] = 'Bogus'
        ex = self.assertRaises(ValueError, receivers.ReceiverCreateRequestBody,
                               **body)
        self.assertEqual("Value 'Bogus' is not acceptable for field 'type'.",
                         six.text_type(ex))

    def test_receiver_create_request_body_invalid_action(self):
        body = copy.deepcopy(self.body)
        body['type'] = 'webhook'
        body['cluster_id'] = 'cluster-01'
        body['action'] = 'Foo'

        ex = self.assertRaises(ValueError, receivers.ReceiverCreateRequestBody,
                               **body)
        self.assertEqual("Value 'Foo' is not acceptable for field 'action'.",
                         six.text_type(ex))

    def test_receiver_create_request(self):
        body = receivers.ReceiverCreateRequestBody(**self.body)
        request = {'receiver': body}
        sot = receivers.ReceiverCreateRequest(**request)
        self.assertIsInstance(sot.receiver,
                              receivers.ReceiverCreateRequestBody)
        self.assertEqual('test-receiver', sot.receiver.name)
        self.assertEqual('message', sot.receiver.type)


class TestReceiverList(test_base.SenlinTestCase):

    def test_receiver_list_request_full(self):
        params = {
            'name': ['receiver01'],
            'type': ['webhook'],
            'action': ['CLUSTER_RESIZE', 'CLUSTER_SCALE_IN'],
            'cluster_id': ['8c3c9af7-d768-4c5a-a21e-5261b22d749d'],
            'user': ['8cbac8cf571b41bd8e27fb1a4bcaa7d7'],
            'limit': 3,
            'marker': 'f1ed0d50-7651-4599-a8cb-c86e9c7123f5',
            'sort': 'name:asc',
            'project_safe': False,
        }
        sot = receivers.ReceiverListRequest(**params)
        self.assertEqual(['receiver01'], sot.name)
        self.assertEqual(['webhook'], sot.type)
        self.assertEqual(['CLUSTER_RESIZE', 'CLUSTER_SCALE_IN'], sot.action)
        self.assertEqual(['8c3c9af7-d768-4c5a-a21e-5261b22d749d'],
                         sot.cluster_id)
        self.assertEqual(['8cbac8cf571b41bd8e27fb1a4bcaa7d7'], sot.user)
        self.assertEqual(3, sot.limit)
        self.assertEqual('f1ed0d50-7651-4599-a8cb-c86e9c7123f5', sot.marker)
        self.assertEqual('name:asc', sot.sort)
        self.assertFalse(sot.project_safe)

    def test_receiver_list_request_default(self):
        sot = receivers.ReceiverListRequest()
        sot.obj_set_defaults()
        self.assertTrue(sot.project_safe)


class TestReceiverGet(test_base.SenlinTestCase):

    def test_receiver_get_request_full(self):
        params = {
            'identity': 'receiver-001'
        }
        sot = receivers.ReceiverGetRequest(**params)
        self.assertEqual('receiver-001', sot.identity)


class TestReceiverUpdate(test_base.SenlinTestCase):
    data = {
        'name': 'receiver01',
        'type': 'webhook',
        'action': 'CLUSTER_SCALE_OUT',
        'params': {'count': '2'},
    }

    def test_receiver_update_request(self):
        sot = receivers.ReceiverUpdateRequest(**self.data)
        self.assertEqual('receiver01', sot.name)
        self.assertEqual('webhook', sot.type)
        self.assertEqual('CLUSTER_SCALE_OUT', sot.action),
        self.assertEqual({'count': '2'}, sot.params)


class TestReceiverDelete(test_base.SenlinTestCase):

    body = {
        'identity': 'test-receiver'
    }

    def test_receiver_delete_request(self):
        sot = receivers.ReceiverDeleteRequest(**self.body)
        self.assertEqual('test-receiver', sot.identity)


class TestReceiverNotify(test_base.SenlinTestCase):

    def test_receiver_notify_request(self):
        params = {
            'identity': 'receiver-001'
        }
        sot = receivers.ReceiverNotifyRequest(**params)
        self.assertEqual('receiver-001', sot.identity)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools

import jsonschema
from oslo_utils import strutils
import six
from webob import exc

from senlin.common.i18n import _
from senlin.common import policy
from senlin.objects import base as obj_base


def policy_enforce(handler):
    """Decorator that enforces policies.

    Checks the path matches the request context and enforces policy defined in
    policy file and policies in code.

    This is a handler method decorator.
    """
    @functools.wraps(handler)
    def policy_checker(controller, req, **kwargs):
        # Enable project_id based target check
        rule = "%s:%s" % (controller.REQUEST_SCOPE,
                          handler.__name__)
        allowed = policy.enforce(context=req.context, rule=rule, target={})
        if not allowed:
            raise exc.HTTPForbidden()
        return handler(controller, req, **kwargs)

    return policy_checker


def parse_request(name, req, body, key=None):
    """Formalizes an API request and validates it.

    :param name: The name for a versioned request object.
    :param req: Reference to a WSGI request object.
    :param body: The JSON body (if any) that accompanies a request. Could be
                 augmented by controller before getting passed here.
    :param key: An optional key indicating the inner object for a request.
    :returns: A validated, versioned request object
    """
    try:
        req_cls = obj_base.SenlinObject.obj_class_from_name(name)
    except Exception as ex:
        raise exc.HTTPBadRequest(six.text_type(ex))

    try:
        primitive = req_cls.normalize_req(name, body, key)
    except ValueError as ex:
        raise exc.HTTPBadRequest(six.text_type(ex))

    version = req_cls.find_version(req.context)
    obj = None
    try:
        obj = req_cls.obj_from_primitive(primitive)
        jsonschema.validate(primitive, obj.to_json_schema())
    except ValueError as ex:
        raise exc.HTTPBadRequest(six.text_type(ex))
    except jsonschema.exceptions.ValidationError as ex:
        raise exc.HTTPBadRequest(six.text_type(ex.message))

    # Do version coversion if necessary
    if obj is not None and version != req_cls.VERSION:
        obj.obj_make_compatible(primitive, version)
        return req_cls.obj_from_primitive(primitive)

    return obj


def get_allowed_params(params, whitelist):
    """Extract from ``params`` all entries listed in ``whitelist``.

    The returning dict will contain an entry for a key if, and only if,
    there's an entry in ``whitelist`` for that key and at least one entry in
    ``params``. If ``params`` contains multiple entries for the same key, it
    will yield an array of values: ``{key: [v1, v2,...]}``

    :param params: a NestedMultiDict from webob.Request.params
    :param whitelist: an array of strings to whitelist

    :returns: a dict with {key: value} pairs
    """
    allowed_params = {}

    for key, get_type in whitelist.items():
        value = None
        if get_type == 'single':
            value = params.get(key)
        elif get_type in ('mixed', 'multi'):
            value = params.getall(key)

        if value:
            allowed_params[key] = value

    return allowed_params


def parse_bool_param(name, value):
    if str(value).lower() not in ('true', 'false'):
        msg = _("Invalid value '%(value)s' specified for '%(name)s'"
                ) % {'name': name, 'value': value}
        raise exc.HTTPBadRequest(msg)

    return strutils.bool_from_string(value, strict=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin.tests.tempest.api import base
from senlin.tests.tempest.common import utils


class TestClusterCreate(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestClusterCreate, self).setUp()
        profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, profile_id)

        self.profile_id = profile_id

    @decorators.idempotent_id('61cbe340-937a-40d5-9d2f-067f2c7cafcc')
    def test_cluster_create_all_attrs_defined(self):
        # Create cluster
        name = 'test-cluster'
        desired_capacity = 2
        min_size = 1
        max_size = 3
        metadata = {'k1': 'v1'}
        timeout = 120
        params = {
            'cluster': {
                'profile_id': self.profile_id,
                'desired_capacity': desired_capacity,
                'min_size': min_size,
                'max_size': max_size,
                'timeout': timeout,
                'metadata': {'k1': 'v1'},
                'name': name
            }
        }
        res = self.client.create_obj('clusters', params)

        # Verify resp of cluster create API
        self.assertEqual(202, res['status'])
        self.assertIsNotNone(res['body'])
        self.assertIn('actions', res['location'])
        cluster = res['body']
        for key in ['created_at', 'data', 'domain', 'id', 'init_at', 'nodes',
                    'policies', 'profile_id', 'profile_name', 'project',
                    'status', 'status_reason', 'updated_at', 'user']:
            self.assertIn(key, cluster)
        self.assertIn(name, cluster['name'])
        self.assertEqual(desired_capacity, cluster['desired_capacity'])
        self.assertEqual(min_size, cluster['min_size'])
        self.assertEqual(max_size, cluster['max_size'])
        self.assertEqual(metadata, cluster['metadata'])
        self.assertEqual(timeout, cluster['timeout'])

        # Wait cluster to be active before moving on
        action_id = res['location'].split('/actions/')[1]
        self.client.wait_for_status('actions', action_id, 'SUCCEEDED')

        self.addCleanup(utils.delete_a_cluster, self, cluster['id'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin.tests.tempest.api import base
from senlin.tests.tempest.common import utils


class TestActionList(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestActionList, self).setUp()
        profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, profile_id)

        cluster_id = utils.create_a_cluster(self, profile_id)
        self.addCleanup(utils.delete_a_cluster, self, cluster_id)

    @decorators.idempotent_id('2e47639b-7f58-4fb4-a147-a8c6bf184e97')
    def test_action_list(self):
        res = self.client.list_objs('actions')

        self.assertEqual(200, res['status'])
        self.assertIsNone(res['location'])
        self.assertIsNotNone(res['body'])

        actions = res['body']
        for action in actions:
            for key in ['action', 'cause', 'created_at', 'data',
                        'depended_by', 'depends_on', 'end_time', 'id',
                        'inputs', 'interval', 'name', 'outputs', 'owner',
                        'start_time', 'status', 'status_reason', 'target',
                        'timeout', 'updated_at']:
                self.assertIn(key, action)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin.tests.tempest.common import constants
from senlin.tests.tempest.common import utils
from senlin.tests.tempest.functional import base


class TestLBPolicy(base.BaseSenlinFunctionalTest):

    def setUp(self):
        super(TestLBPolicy, self).setUp()
        self.profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, self.profile_id)
        self.cluster_id = utils.create_a_cluster(self, self.profile_id,
                                                 min_size=0, max_size=5,
                                                 desired_capacity=2)
        self.addCleanup(utils.delete_a_cluster, self, self.cluster_id)

    @decorators.attr(type=['functional'])
    @decorators.idempotent_id('6b513a5d-75b6-447a-b95d-e17b84ac9ee8')
    def test_lb_policy(self):
        # Verify there is no lb information in node data
        cluster = utils.get_a_cluster(self, self.cluster_id)
        for n in cluster['nodes']:
            node = utils.get_a_node(self, n)
            self.assertNotIn('lb_member', node['data'])

        # Create a lb policy
        spec = constants.spec_lb_policy
        policy_id = utils.create_a_policy(self, spec)
        lb_policy = utils.get_a_policy(self, policy_id)
        self.addCleanup(utils.delete_a_policy, self, lb_policy['id'])

        # Attach lb policy to cluster
        utils.cluster_attach_policy(self, self.cluster_id, lb_policy['id'])

        # Verify lb information is added into node data
        cluster = utils.get_a_cluster(self, self.cluster_id)
        for n in cluster['nodes']:
            node = utils.get_a_node(self, n)
            self.assertIn('lb_member', node['data'])

        # Scale out cluster
        utils.cluster_scale_out(self, self.cluster_id)

        # Verify scale out result
        cluster = utils.get_a_cluster(self, self.cluster_id)
        self.assertEqual('ACTIVE', cluster['status'])
        self.assertEqual(3, cluster['desired_capacity'])
        self.assertEqual(3, len(cluster['nodes']))

        # Verify lb information in all nodes' data
        cluster = utils.get_a_cluster(self, self.cluster_id)
        for n in cluster['nodes']:
            node = utils.get_a_node(self, n)
            self.assertIn('lb_member', node['data'])

        # Scale in cluster
        utils.cluster_scale_in(self, self.cluster_id)

        # Verify scale in result
        cluster = utils.get_a_cluster(self, self.cluster_id)
        self.assertEqual('ACTIVE', cluster['status'])
        self.assertEqual(2, cluster['desired_capacity'])
        self.assertEqual(2, len(cluster['nodes']))
        for n in cluster['nodes']:
            node = utils.get_a_node(self, n)
            self.assertIn('lb_member', node['data'])

        # Detach lb policy from cluster
        utils.cluster_detach_policy(self, self.cluster_id, lb_policy['id'])

        # Verify lb information has been removed from node data
        cluster = utils.get_a_cluster(self, self.cluster_id)
        for n in cluster['nodes']:
            node = utils.get_a_node(self, n)
            self.assertNotIn('lb_member', node['data'])
